@@ -31,8 +31,9 @@ class Puppet::FileBucket::Dipper
 
   # Back up a file to our bucket
   def backup(file)
-    raise(ArgumentError, "File #{file} does not exist") unless ::File.exist?(file)
-    contents = IO.binread(file)
+    file_handle = Puppet::FileSystem::File.new(file)
+    raise(ArgumentError, "File #{file} does not exist") unless file_handle.exist?
+    contents = file_handle.binread
     begin
       file_bucket_file = Puppet::FileBucket::File.new(contents, :bucket_path => @local_path)
       files_original_path = absolutize_path(file)
@@ -65,8 +66,9 @@ class Puppet::FileBucket::Dipper
   # Restore the file
   def restore(file,sum)
     restore = true
-    if FileTest.exists?(file)
-      cursum = Digest::MD5.hexdigest(IO.binread(file))
+    file_handle = Puppet::FileSystem::File.new(file)
+    if file_handle.exist?
+      cursum = Digest::MD5.hexdigest(file_handle.binread)
 
       # if the checksum has changed...
       # this might be extra effort
@@ -79,8 +81,8 @@ class Puppet::FileBucket::Dipper
       if newcontents = getfile(sum)
         newsum = Digest::MD5.hexdigest(newcontents)
         changed = nil
-        if FileTest.exists?(file) and ! FileTest.writable?(file)
-          changed = ::File.stat(file).mode
+        if file_handle.exist? and ! file_handle.writable?
+          changed = Puppet::FileSystem::File.new(file).stat.mode
           ::File.chmod(changed | 0200, file)
         end
         ::File.open(file, ::File::WRONLY|::File::TRUNC|::File::CREAT) { |of|
