@@ -7,61 +7,6 @@ class Puppet::Parser::TypeLoader
   extend  Forwardable
   include Puppet::Node::Environment::Helper
 
-  # Helper class that makes sure we don't try to import the same file
-  # more than once from either the same thread or different threads.
-  class Helper
-    include MonitorMixin
-    def initialize
-      super
-      # These hashes are indexed by filename
-      @state = {} # :doing or :done
-      @thread = {} # if :doing, thread that's doing the parsing
-      @cond_var = {} # if :doing, condition var that will be signaled when done.
-    end
-
-    # Execute the supplied block exactly once per file, no matter how
-    # many threads have asked for it to run.  If another thread is
-    # already executing it, wait for it to finish.  If this thread is
-    # already executing it, return immediately without executing the
-    # block.
-    #
-    # Note: the reason for returning immediately if this thread is
-    # already executing the block is to handle the case of a circular
-    # import--when this happens, we attempt to recursively re-parse a
-    # file that we are already in the process of parsing.  To prevent
-    # an infinite regress we need to simply do nothing when the
-    # recursive import is attempted.
-    def do_once(file)
-      need_to_execute = synchronize do
-        case @state[file]
-        when :doing
-          if @thread[file] != Thread.current
-            @cond_var[file].wait
-          end
-          false
-        when :done
-          false
-        else
-          @state[file] = :doing
-          @thread[file] = Thread.current
-          @cond_var[file] = new_cond
-          true
-        end
-      end
-      if need_to_execute
-        begin
-          yield
-        ensure
-          synchronize do
-            @state[file] = :done
-            @thread.delete(file)
-            @cond_var.delete(file).broadcast
-          end
-        end
-      end
-    end
-  end
-
   # Import manifest files that match a given file glob pattern.
   #
   # @param pattern [String] the file glob to apply when determining which files
@@ -103,7 +48,6 @@ class Puppet::Parser::TypeLoader
 
   def initialize(env)
     self.environment = env
-    @loading_helper = Helper.new
   end
 
   # Try to load the object with the given fully qualified name.
@@ -148,11 +92,11 @@ class Puppet::Parser::TypeLoader
   end
 
   def load_files(modname, files)
+    @loaded ||= {}
     loaded_asts = []
-    files.each do |file|
-      @loading_helper.do_once(file) do
-        loaded_asts << parse_file(file)
-      end
+    files.reject { |file| @loaded[file] }.each do |file|
+      loaded_asts << parse_file(file)
+      @loaded[file] = true
     end
 
     loaded_asts.collect do |ast|
