@@ -7,24 +7,20 @@ class Puppet::Provider::AixObject < Puppet::Provider
   desc "Generic AIX resource provider"
 
   # The real provider must implement these functions.
-  def lscmd(value=@resource[:name])
-    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: #{detail}"
+  def lscmd( _value = @resource[:name] )
+    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: Base AixObject provider doesn't implement lscmd"
   end
 
-  def lscmd(value=@resource[:name])
-    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: #{detail}"
+  def addcmd( _extra_attrs = [] )
+    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: Base AixObject provider doesn't implement addcmd"
   end
 
-  def addcmd(extra_attrs = [])
-    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: #{detail}"
-  end
-
-  def modifycmd(attributes_hash)
-    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: #{detail}"
+  def modifycmd( _attributes_hash = {} )
+    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: Base AixObject provider doesn't implement modifycmd"
   end
 
   def deletecmd
-    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: #{detail}"
+    raise Puppet::Error, "Method not defined #{@resource.class.name} #{@resource.name}: Base AixObject provider doesn't implement deletecmd"
   end
 
   # Valid attributes to be managed by this provider.
@@ -152,7 +148,7 @@ class Puppet::Provider::AixObject < Puppet::Provider
   # Parse AIX command attributes from the output of an AIX command, that
   # which format is a list of space separated of key=value pairs:
   # "uid=100 groups=a,b,c".
-  # It returns an hash.
+  # It returns a hash.
   #
   # If a mapping is provided, the keys are translated as defined in the
   # mapping hash. And only values included in mapping will be added
@@ -161,7 +157,7 @@ class Puppet::Provider::AixObject < Puppet::Provider
   def parse_attr_list(str, mapping=self.class.attribute_mapping_from)
     properties = {}
     attrs = []
-    if !str or (attrs = str.split()).empty?
+    if str.nil? or (attrs = str.split()).empty?
       return nil
     end
 
@@ -169,11 +165,13 @@ class Puppet::Provider::AixObject < Puppet::Provider
       if i.include? "=" # Ignore if it does not include '='
         (key_str, val) = i.split('=')
         # Check the key
-        if !key_str or key_str.empty?
+        if key_str.nil? or key_str.empty?
           info "Empty key in string 'i'?"
           continue
         end
+        key_str.strip!
         key = key_str.to_sym
+        val.strip! if val
 
         properties = self.load_attribute(key, val, mapping, properties)
       end
@@ -193,7 +191,7 @@ class Puppet::Provider::AixObject < Puppet::Provider
   def parse_colon_list(str, key_list, mapping=self.class.attribute_mapping_from)
     properties = {}
     attrs = []
-    if !str or (attrs = str.split(':')).empty?
+    if str.nil? or (attrs = str.split(':')).empty?
       return nil
     end
 
@@ -227,9 +225,9 @@ class Puppet::Provider::AixObject < Puppet::Provider
       # Execute lsuser, split all attributes and add them to a dict.
       begin
         output = execute(self.lscmd)
-        @objectinfo = self.parse_command_output(execute(self.lscmd))
+        @objectinfo = self.parse_command_output(output)
         # All attributtes without translation
-        @objectosinfo = self.parse_command_output(execute(self.lscmd), nil)
+        @objectosinfo = self.parse_command_output(output, nil)
       rescue Puppet::ExecutionFailure => detail
         # Print error if needed. FIXME: Do not check the user here.
         Puppet.debug "aix.getinfo(): Could not find #{@resource.class.name} #{@resource.name}: #{detail}"
@@ -241,10 +239,10 @@ class Puppet::Provider::AixObject < Puppet::Provider
   # Like getinfo, but it will not use the mapping to translate the keys and values.
   # It might be usefult to retrieve some raw information.
   def getosinfo(refresh = false)
-    if @objectosinfo .nil? or refresh == true
+    if @objectosinfo.nil? or refresh == true
       getinfo(refresh)
     end
-    @objectosinfo
+    @objectosinfo || Hash.new
   end
 
 
@@ -290,7 +288,7 @@ class Puppet::Provider::AixObject < Puppet::Provider
   # providers, preferably with values already filled in, not resources.
   def self.instances
     objects=[]
-    self.list_all.each { |entry|
+    list_all.each { |entry|
       objects << new(:name => entry, :ensure => :present)
     }
     objects
@@ -383,7 +381,7 @@ class Puppet::Provider::AixObject < Puppet::Provider
     end
 
     # Refresh de info.
-    hash = getinfo(true)
+    getinfo(true)
   end
 
   def initialize(resource)
