@@ -8,7 +8,11 @@ module Puppet
       on the value of the 'ensure' parameter.
 
       Note that if a `mount` receives an event from another resource,
-      it will try to remount the filesystems if `ensure` is set to `mounted`."
+      it will try to remount the filesystems if `ensure` is set to `mounted`.
+
+      **Autorequires:** If Puppet is managing any parents of a mount resource ---
+      that is, other mount points higher up in the filesystem --- the child
+      mount will autorequire them."
 
     feature :refreshable, "The provider can remount the filesystem.",
       :methods => [:remount]
@@ -268,5 +272,15 @@ module Puppet
         return property.value
       end
     end
+
+    # Ensure that mounts higher up in the filesystem are mounted first
+    autorequire(:mount) do
+      dependencies = []
+      Pathname.new(@parameters[:name].value).ascend do |parent|
+        dependencies.unshift parent.to_s
+      end
+      dependencies[0..-2]
+    end
+
   end
 end
