@@ -565,29 +565,32 @@ describe Puppet::Type.type(:scheduled_task).provider(:win32_taskscheduler), :if
     end
   end
 
-  describe '#user_insync?' do
+  describe '#user_insync?', :if => Puppet.features.microsoft_windows? do
     let(:resource) { described_class.new(:name => 'foobar', :command => 'C:\Windows\System32\notepad.exe') }
 
-    before :each do
-      Puppet::Util::ADSI.stubs(:sid_for_account).with('system').returns('SYSTEM SID')
-      Puppet::Util::ADSI.stubs(:sid_for_account).with('joe').returns('SID A')
-      Puppet::Util::ADSI.stubs(:sid_for_account).with('MACHINE\joe').returns('SID A')
-      Puppet::Util::ADSI.stubs(:sid_for_account).with('bob').returns('SID B')
-    end
-
     it 'should consider the user as in sync if the name matches' do
+      Puppet::Util::Windows::Security.expects(:name_to_sid).with('joe').twice.returns('SID A')
+
       resource.should be_user_insync('joe', ['joe'])
     end
 
     it 'should consider the user as in sync if the current user is fully qualified' do
+      Puppet::Util::Windows::Security.expects(:name_to_sid).with('joe').returns('SID A')
+      Puppet::Util::Windows::Security.expects(:name_to_sid).with('MACHINE\joe').returns('SID A')
+
       resource.should be_user_insync('MACHINE\joe', ['joe'])
     end
 
     it 'should consider a current user of the empty string to be the same as the system user' do
+      Puppet::Util::Windows::Security.expects(:name_to_sid).with('system').twice.returns('SYSTEM SID')
+
       resource.should be_user_insync('', ['system'])
     end
 
     it 'should consider different users as being different' do
+      Puppet::Util::Windows::Security.expects(:name_to_sid).with('joe').returns('SID A')
+      Puppet::Util::Windows::Security.expects(:name_to_sid).with('bob').returns('SID B')
+
       resource.should_not be_user_insync('joe', ['bob'])
     end
   end
@@ -1456,7 +1459,7 @@ describe Puppet::Type.type(:scheduled_task).provider(:win32_taskscheduler), :if
       end
     end
 
-    describe '#user=' do
+    describe '#user=', :if => Puppet.features.microsoft_windows? do
       before :each do
         @mock_task = mock
         @mock_task.responds_like(Win32::TaskScheduler.new)
@@ -1466,7 +1469,7 @@ describe Puppet::Type.type(:scheduled_task).provider(:win32_taskscheduler), :if
       end
 
       it 'should use nil for user and password when setting the user to the SYSTEM account' do
-        Puppet::Util::ADSI.stubs(:sid_for_account).with('system').returns('SYSTEM SID')
+        Puppet::Util::Windows::Security.stubs(:name_to_sid).with('system').returns('SYSTEM SID')
 
         resource = Puppet::Type.type(:scheduled_task).new(
           :name    => 'Test Task',
@@ -1480,7 +1483,7 @@ describe Puppet::Type.type(:scheduled_task).provider(:win32_taskscheduler), :if
       end
 
       it 'should use the specified user and password when setting the user to anything other than SYSTEM' do
-        Puppet::Util::ADSI.stubs(:sid_for_account).with('my_user_name').returns('SID A')
+        Puppet::Util::Windows::Security.stubs(:name_to_sid).with('my_user_name').returns('SID A')
 
         resource = Puppet::Type.type(:scheduled_task).new(
           :name     => 'Test Task',
