@@ -10,7 +10,7 @@ describe Puppet::Util::RunMode do
     @run_mode.run_dir.should == '$vardir/run'
   end
 
-  describe Puppet::Util::UnixRunMode do
+  describe Puppet::Util::UnixRunMode, :unless => Puppet.features.microsoft_windows? do
     before do
       @run_mode = Puppet::Util::UnixRunMode.new('fake')
     end
@@ -59,17 +59,9 @@ describe Puppet::Util::RunMode do
         end
       end
     end
-
-    def without_home
-      saved_home = ENV["HOME"]
-      ENV.delete "HOME"
-      yield
-    ensure
-      ENV["HOME"] = saved_home
-    end
   end
 
-  describe Puppet::Util::WindowsRunMode do
+  describe Puppet::Util::WindowsRunMode, :if => Puppet.features.microsoft_windows? do
     before do
       if not Dir.const_defined? :COMMON_APPDATA
         Dir.const_set :COMMON_APPDATA, "/CommonFakeBase"
@@ -92,6 +84,18 @@ describe Puppet::Util::RunMode do
       it "has confdir in ~/.puppet when run as non-root" do
         as_non_root { @run_mode.conf_dir.should == File.expand_path("~/.puppet") }
       end
+
+      it "fails when asking for the conf_dir as non-root and there is no %HOME%, %HOMEDRIVE%, and %USERPROFILE%" do
+        as_non_root do
+          without_env('HOME') do
+            without_env('HOMEDRIVE') do
+              without_env('USERPROFILE') do
+                expect { @run_mode.conf_dir }.to raise_error ArgumentError, /couldn't find HOME/
+              end
+            end
+          end
+        end
+      end
     end
 
     describe "#var_dir" do
@@ -102,6 +106,18 @@ describe Puppet::Util::RunMode do
       it "has vardir in ~/.puppet/var when run as non-root" do
         as_non_root { @run_mode.var_dir.should == File.expand_path("~/.puppet/var") }
       end
+
+      it "fails when asking for the conf_dir as non-root and there is no %HOME%, %HOMEDRIVE%, and %USERPROFILE%" do
+        as_non_root do
+          without_env('HOME') do
+            without_env('HOMEDRIVE') do
+              without_env('USERPROFILE') do
+                expect { @run_mode.var_dir }.to raise_error ArgumentError, /couldn't find HOME/
+              end
+            end
+          end
+        end
+      end
     end
   end
 
@@ -114,4 +130,16 @@ describe Puppet::Util::RunMode do
     Puppet.features.stubs(:root?).returns(false)
     yield
   end
+
+  def without_env(name, &block)
+    saved = ENV[name]
+    ENV.delete name
+    yield
+  ensure
+    ENV[name] = saved
+  end
+
+  def without_home(&block)
+    without_env('HOME', &block)
+  end
 end
